@@ -54,7 +54,7 @@ class DynamoFL(_Base):
             not_sampled = False
             if 'notSampled' in j['data']:
                 not_sampled = j['data']['notSampled']
-                
+
 
             self.project_participants.append({
                 'project_key': project_key,
@@ -82,8 +82,9 @@ class DynamoFL(_Base):
                 samples = j['data']['samples']
 
             for p in self.project_participants:
-                if project_key == p['project_key'] and p['datasource_key'] in samples:
-                    self.train_and_test_callback(p['datasource_key'], j['data']['project'])
+                if project_key == p['project_key']:
+                    if not len(samples) or p['datasource_key'] in samples:
+                        self.train_and_test_callback(p['datasource_key'], j['data']['project'])
 
         elif j['event'] == 'hyperparams-updated':
             for p in self.project_participants:
@@ -170,7 +171,7 @@ class DynamoFL(_Base):
         prev_round = self._get_last_fed_model_round(current_round, project_info['isComplete'])
         federated_model_path = get_federated_path(project_key, model_path, ext, datasource_key, prev_round)
 
-        yes_stats = len(self._check_stats(project_key, datasource_key, prev_round))
+        yes_stats = self._check_stats(project_info, datasource_key, prev_round)
         yes_submission = self._check_submissions(project_info, datasource_key, current_round)
 
         if not yes_submission or not yes_stats:
@@ -311,14 +312,24 @@ class DynamoFL(_Base):
         else:
             return len(user_submissions)
 
-
-    def _check_stats(self, project_key, datasource_key, round):
+    def _check_stats(self, project_info, datasource_key, round):
         params = {
             'owned': True,
             'datasource': datasource_key,
             'round': round
         }
-        return self._make_request('GET', f'/projects/{project_key}/stats', params, list=True)
+        project_key = project_info['key']
+        user_stats = self._make_request('GET', f'/projects/{project_key}/stats', params, list=True)
+        # if sampled project, check if round reached full size
+        if 'clientSampleSize' in project_info and project_info['clientSampleSize']:
+            all_stats = self._make_request('GET', f'/projects/{project_key}/stats', { 'round': round }, list=True)
+            if len(all_stats) < project_info['clientSampleSize']:
+                return False
+            else:
+                return True
+        else:
+            return len(user_stats)
+
 
 
 def get_trained_path(project_key, base, ext, ds, round):
